package Tk::Workspace;

=head1 NAME

Workspace - Generate embedded perl workspace file-window-script
            via Tk (alpha version).

=head1 SYNOPSIS

    use Workspace;
    use Tk;
    use IO::File;

    Workspace::open(Workspace::create("workspace"));

=head1 DESCRIPTION

Workspace uses the Tk::Text widget to create an embedded perl
text editor.  The resulting file can be run as a standalone
program.  

=head1 AUTHOR

Robert Kiesling, rkiesling@mainmatter.com

=head1 TODO

Lots of stuff.  Subclass the text widget to provide embedded
text-processing language.  Subclass the Tk::Top widget so that the
window gets notified if the file is changed.  

=head1 VERSION

$Id: Workspace.pm,v 0.56 1999/12/27 16:16:49 kiesling Exp $

=head1 REFERENCES

Perltoot.* by Tom Christiansen.
Programming Perl, by L. Wall, T. Christiansen, and R. L. Schwartz

=cut


use Tk;
use Tk::MainWindow;
use Tk::TextUndo;
use Tk::Entry;
use FileHandle;
use IO::File;

@ISA = qw(Tk::Widget);

use Tk qw(Ev);

use base qw(Tk::Widget Tk::MainWindow);

Construct Tk::Widget 'Workspace';

# class variables

my $text=''; # This is the initial text.

my @Workspaceobject = 
    ('#!/usr/bin/perl',
     'my $text=\'\';',
     'my $name=\'\';',
     'use Tk::Workspace;', 
     '@ISA = qw(Tk::Window);',
     'use strict;',
     'use Tk;',
     'use FileHandle;',
     'use Env qw(HOME);',
     'my $workspace = Tk::Workspace -> new;',
     '$workspace -> name($name);',
     '$workspace -> menubar -> pack (-anchor => \'w\', -fill => \'x\');',
     '$workspace -> textspace -> insert ( \'end\', $text );',
     '$workspace -> textspace -> pack;',
     '$workspace -> bind;',
     'MainLoop;' );

my $defaultbackgroundcolor="white";
my $defaulttextfont="*-courier-medium-r-*-*-12-*";
my $menufont="*-helvetica-medium-r-*-*-12-*";
my $Perlhomedirectory='.perlobjects';
my $Default=$HOME . '/' . $Perlhomedirectory . '/' . '.default';
my $SystemCopyCommand='cp';       # OS-specific file copy command.

my $clipboard;          # Internal clipboard.

sub new {
    my $proto = shift;
    my $class = ref( $proto ) || $proto;
    my $self = {
	window => new MainWindow,
	textspace => undef,
	name => ((@_)?@_:'Workspace'),
	textfont => $defaulttextfont,
	backgroundcolor => $defaultbackgroundcolor,
	UNDO => [],  # undo stack for TextUndo module.
	filemenu => undef,
	editmenu => undef,
	helpmenu => undef,
	menubar => undef,
	text => []                   # The text itself.
	};
    bless($self, $class);
    $self -> {textspace} = ($self -> {window}) -> 
	    TextUndo ( -font => $defaulttextfont,
		   -background => $defaultbackgroundcolor,
		   -width => 80,
		   -height => 25,
		   -exportselection => 'true',
		   -wrap => 'word');
    $self -> {menubar} = ($self -> {window} ) -> 
		    Menu ( -type => 'menubar',
			   -font => $menufont );

    $self -> {filemenu} = ($self -> {menubar}) -> Menu;
    $self -> {editmenu} = ($self -> {menubar}) -> Menu;
    ($self -> {helpmenu}) = ($self -> {menubar}) -> Menu;

    $self -> {menubar}  -> 
	add ('cascade',
	     -font => $menufont,
	     -label => 'File',
	     -menu => $self -> {filemenu} );
    $self -> {menubar}  -> 
	add ('cascade',
	     -font => $menufont,
	     -label => 'Edit',
	     -menu => $self -> {editmenu} );
    $self -> {menubar} -> add ('separator');
    $self -> {menubar}  -> 
	add ('cascade',
	     -font => $menufont,
	     -label => 'Help',
	     -menu => $self -> {helpmenu} );
    $self -> {filemenu} -> add ( 'command', -label => 'Import Text...',
				 -font => $menufont,
				 -state => 'normal',
				 -command => sub{$self -> ws_import});
    $self -> {filemenu} -> add ( 'command', -label => 'Export Text...',
				 -font => $menufont,
				 -state => 'normal',
				 -command => sub{$self -> ws_export});
    $self -> {filemenu} -> add ('separator');
    $self -> {filemenu} -> add ( 'command', -label => 'Quit',
				 -font => $menufont,
				 -state => 'normal',
				 -accelerator => 'C-q',
				 -command => sub{$self -> ws_close});
    $self -> {editmenu} -> add ( 'command', -label => 'Cut',
				 -font => $menufont,
				 -state => 'normal',
				 -accelerator => 'C-x',
				 -command => sub{$self -> ws_cut});
    $self -> {editmenu} -> add ( 'command', -label => 'Copy',
				 -font => $menufont,
				 -accelerator => 'C-c',
				 -state => 'normal',
				 -command => sub{$self -> ws_copy});
    $self -> {editmenu} -> add ( 'command', -label => 'Paste',
				 -font => $menufont,
				 -accelerator => 'C-v',
				 -state => 'normal',
				 -command => sub{$self -> ws_paste});
    $self -> {editmenu} -> add ('separator');
    $self -> {editmenu} -> add ( 'command', -label => 'Undo',
				 -font => $menufont,
				 -state => 'normal',
				 -accelerator => 'C-u',
				 -command => sub{$self -> ws_undo});
    $self -> {helpmenu} -> add ( 'command', -label => 'About...',
				 -font => $menufont,
				 -state => 'normal',
				 -accelerator => 'C-a',
				 -command => sub{$self -> ws_about});
    return $self;
}



###
### Instance variable methods.
###

sub bind {

    my $self = shift;

    ($self -> window) -> SUPER::bind('<Control-q>', 
				    sub{$self -> ws_close});
    ($self -> window) -> SUPER::bind('<Control-x>', 
				    sub{$self -> ws_cut});
    ($self -> window) -> SUPER::bind('<Control-c>', 
				    sub{$self -> ws_copy});
    ($self -> window) -> SUPER::bind('<Control-v>', 
				    sub{$self -> ws_paste});
    ($self -> window) -> SUPER::bind('<Control-u>', 
				    sub{$self -> ws_undo});
    ($self -> window) -> SUPER::bind('<Control-a>', 
				    sub{$self -> ws_about});
}

sub window {
    my $self = shift;
    if (@_) { $self -> {window} = shift }
    return $self -> {window}
}

sub textspace {
    my $self = shift;
    if (@_) { $self -> {textspace} = shift }
    return $self -> {textspace}
}

sub name {
    my $self = shift;
    if (@_) { $self -> {name} = shift }
    return $self -> {name}
}

sub textfont {
    my $self = shift;
    if (@_) { $self -> {textfont} = shift }
    return $self -> {textfont}
}

sub menubar {
    my $self = shift;
    if (@_) { $self -> {menubar} = shift }
    return $self -> {menubar}
}

sub filemenu {
    my $self = shift;
    if (@_) { $self -> {filemenu} = shift }
    return $self -> {filemenu}
}

sub editmenu {
    my $self = shift;
    if (@_) { $self -> {editmenu} = shift }
    return $self -> {editmenu}
}

sub open {
    my ($name) = @_;

    my @command_line = ( "\./" . $name . ' &');
    system( @command_line );
}

sub ws_close {
    my $self = shift;
    my $dir = $HOME . '/' . $Perlhomedirectory;
    my $workspacename = $self -> name;
    my $workspacepath = $workspacename;
    my $tmppath = $workspacepath . ".tmp";
    my $contents;
    my $object;
    open FILE, ">>" . $tmppath;
    $contents = ($self -> textspace) -> get( '1.0', end );
    printf FILE '#!/usr/bin/perl' . "\n";

    # concatenate text.
    printf FILE 'my $text = <<\'end-of-text\';' . "\n";
    printf FILE $contents;
    printf FILE "end-of-text\n";

    # This re-creates on the default workspace object, except
    # the first line, the name, and the initial empty text.;
    my @tmpobject = @Workspaceobject;
    grep { s/name\=\'\'/name=\'$workspacename\'/ } @tmpobject;
    grep { s/#!\/usr\/bin\/perl// } @tmpobject;
	   grep { s/my \$text=\'\'\;// } @tmpobject;
	   foreach $line ( @tmpobject ) { printf FILE $line . "\n"; };
	   close FILE;
	   my @remove_old = ( 'mv', $tmppath, $workspacepath );
	   system( @remove_old );
	   chmod 0755, $workspacepath;
    exit();
}

# Create a new Workspace executable if one doesn't exist.
sub create {
    my ($workspacename) = ((@_)?@_:'Workspace');
    my $Source;
    my $directory = ''; # Where are we.

    # Make sure a workspace executable of the same basename
    # doesn't exist already.  If it does, append a .2 to it.
    if ( -e $workspacename ) {
	$workspacename = $workspacename . '.2';
    }

    # try again.
    #Name the workspace...
    my @tmpobject = @Workspaceobject;
    grep { s/name\=\'\'/name\=\'$workspacename\'/ } @tmpobject;

    open FILE, ">" . $workspacename 
	or die "Can't open Workspace " . $workspacename;
    # This creates on the default workspace object.

    foreach $line ( @tmpobject ) { printf FILE $line . "\n"; }
    close FILE;
    chmod 0755, $workspacename;
    return( $workspacename );
}

sub ws_copy {
    my $self = shift;
    my $selection;
    if ( ! (($self -> {textspace}) -> tagRanges('sel')) ) { return; }
    # per clipboard.txt, this asserts workspace text widget's 
    # ownership of X display clipboard, and clears it.
    ($self -> {textspace}) -> clipboardClear;
    $selection = ($self -> {textspace}) 
	-> SelectionGet(-selection => 'PRIMARY',
			-type => 'STRING' );
    # Appends PRIMARY selection to X display clipboard.
    ($self -> {textspace}) -> clipboardAppend($selection);
    $clipboard = $selection;   # our  clipboard, not X's.
    return $selection;
}

sub ws_cut {
    my $self = shift;
    my $selection;
    if ( ! (($self -> {textspace}) -> tagRanges('sel')) ) { return; }
    # per clipboard.txt, this asserts workspace text widget's 
    # ownership of X display clipboard, and clears it.
    ($self -> {textspace}) -> clipboardClear;
    $selection = ($self -> {textspace}) 
	-> SelectionGet(-selection => 'PRIMARY',
			-type => 'STRING' );
    # Appends PRIMARY selection to X display clipboard.
    ($self -> {textspace}) -> clipboardAppend($selection);
    ($self ->{textspace}) -> 
	delete(($self -> {textspace}) -> tagRanges('sel'));
    $clipboard = $selection;   # our  clipboard, not X's.
    return $selection;
}

sub ws_paste {
    my $self = shift;
    my $selection;
    my $point;
    # Don't use CLIPBOARD because of a bug? in PerlTk...
    #
    # Checks PRIMARY selection, then X display clipboard, 
    # and returns if neither is defined.
#    ($self -> {textspace}) -> 
#	selectionOwn(-selection => 'CLIPBOARD');
#    if ( ! (($self -> {textspace}) -> tagRanges('sel')) 
#	 or (($selection =  ($self -> {textspace}) 
#	-> SelectionGet(-selection => 'PRIMARY',
#			-type => 'STRING')) == '') ) {
#	return; 
#    }
#    if ($self -> {textspace} -> tagRanges('sel')) {
#	$selection = ($self -> {textspace}) 
#	    -> SelectionGet(-selection => 'PRIMARY',
#			    -type => 'STRING');
#    } else {
#	$selection = $clipboard;
#    }
    $ selection = ($self -> {textspace}) -> clipboardGet;
    $point = ($self -> {textspace}) -> index("insert");
    ($self -> {textspace}) -> insert( $point,
				      $selection);
    return $selection;
}

sub ws_undo {
    my $self = shift;
    my $undo;
    $undo = ($self -> {textspace}) -> undo;
    return $self
}

sub ws_about {
    my $self = shift;
    my $aboutdialog;
    my $title_text;
    my $version_text;
    my $name_text;
    my $mod_time;
    my $line_space;  # blank label as separator.
    my @filestats = { $device,
		    $inode,
		    $nlink,
		    $uid,
		    $gid,
		    $raw_device,
		    $size,
		    $atime,
		    $mtime,
		    $ctime,
		    $blksize,
		    $blocks };
    
    @filestats = stat ($self -> {name});

    $aboutdialog = 
	($self -> {window}) -> 
	    DialogBox( -buttons => ["Ok"]);
    $title_text = $aboutdialog -> add ('Label');
    $version_text = $aboutdialog -> add ('Label');
    $name_text = $aboutdialog -> add ('Label');
    $mod_time = $aboutdialog -> add ('Label');
    $line_space = $aboutdialog -> add ('Label');

    $title_text -> configure ( -font => $menufont,
			       -text => 
	       'Workspace.pm by rkieslin@mainmatter.com <Robert Kiesling>' );
    $version_text -> configure ( -font => $menufont,
				 -text => 'Version $Revision: 0.56 $');
    $name_text -> configure ( -font => $menufont,
                              -text => "\'" . $self -> {name} . "\'" );
    $mod_time -> configure ( -font => $menufont,
                             -text => 'Last modified: ' . 
                             localtime($filestats[8])  );
    $line_space -> configure ( -font =>$menufont,
                               -text => '');

    $name_text -> pack;
    $mod_time -> pack;
    $line_space -> pack;
    $title_text -> pack;
    $version_text -> pack;
    $aboutdialog -> Show;
}

sub ws_import {

    my $self = shift;
    my $import;
    my $filedialog;
    my $filename; 
    my $fh = new IO::File;
    my $nofiledialog;
    
    $filedialog = ($self -> {window}) 
	-> FileSelect ( -directory => '.');
    $filename = $filedialog -> Show;

    $fh -> open( "< $filename" ) or &filenotfound($self);

    while ( read $fh, $import, 512 ) {
	($self -> {textspace}) -> insert ( 'end', $import  );
    }
    close $fh;
}

sub ws_export {
    my $self = shift;
    my $filedialog;
    my $filename;
    my $filename;
    my $fh = new IO::File;

    $filedialog = ($self -> {window})
	-> FileSelect ( -directory => '.' );
    $filename = $filedialog -> Show;

    $fh -> open( "+> $filename" ) or &filenotfound( $self );

    print $fh ($self -> {textspace}) -> get( '1.0', 'end' );

    close $fh;
}

sub filenotfound {

    my $self = shift;

    my $nofiledialog = 
	($self -> {window}) ->
		DialogBox( -buttons => ["OK"]);
    my $filenotfound = $nofiledialog -> add ( 'Label'); 
    $filenotfound -> configure ( -font => $menufont,
			   -text => 'File not found.');
    $filenotfound -> pack;
    $nofiledialog -> Show;
}

sub my_directory {
    open PATHNAME, "pwd |";
    read PATHNAMR, $directory, 512;
    close PATHNAME;
}

1;
__END__
